@@ -26,7 +26,7 @@ from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear
     set_git_meta, set_python_dependencies, set_run_name, set_model_json_path, set_visual_json_path, set_model_file_path, set_prefix_path, set_best_model_joblib_path, _set_result_path, set_shap_json_path
 from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
 from Accuinsight.modeler.core.LcConst import LcConst
-from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook
+from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook, get_os_env
 from Accuinsight.modeler.clients.modeler_api import LifecycleRestApi
 
 from Accuinsight.modeler.core.feature_contribution import shap_value
@@ -299,9 +299,10 @@ class accuinsight(object):
             delta_ts = end_time - start_time
 
             clear_runs(start_time, end_time, delta_ts)
-            modeler_rest = LifecycleRestApi(LcConst.BACK_END_API_URL,
-                                            LcConst.BACK_END_API_PORT,
-                                            LcConst.BACK_END_API_URI)
+            env_value = get_os_env('ENV')
+            modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
+                                            env_value[LcConst.BACK_END_API_PORT],
+                                            env_value[LcConst.BACK_END_API_URI])
             modeler_rest.lc_create_run()
 
 
