@@ -10,8 +10,8 @@ import logging
 import boto3
 from pathlib import Path
 import numpy as np
-from slack import WebClient
-from slack.errors import SlackApiError
+from slack_sdk import WebClient
+from slack_sdk.errors import SlackApiError
 import keras
 from keras.callbacks import CSVLogger
 from Accuinsight.modeler.core import func, path, get
@@ -23,8 +23,7 @@ from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear
 from Accuinsight.modeler.utils.dl_utils import delete_files_except_best, get_best_model_path
 from Accuinsight.modeler.utils.runs_utils import get_aws_info, ProgressPercentage
 from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
-#from Accuinsight.modeler.core.notifier import notifierActor
-from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook
+from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook, get_os_env
 from Accuinsight.modeler.clients.modeler_api import LifecycleRestApi
 
 logging.basicConfig(level=logging.INFO,
@@ -325,17 +324,18 @@ class accuinsight(object):
                 clear_runs(start_ts, end_ts, delta_ts)
                 accuinsight._send_message(metric = self.monitor,
                                           current_value = self.current_value,
-                                          message = message,
-                                          thresholds = thresholds,
-                                          token = token,
-                                          channel_id = channel_id)
-                modeler_rest = LifecycleRestApi(LcConst.BACK_END_API_URL,
-                                                LcConst.BACK_END_API_PORT,
-                                                LcConst.BACK_END_API_URI)
+                                          message = self.message,
+                                          thresholds = self.thresholds,
+                                          token = self.token,
+                                          channel_id = self.channel_id)
+                env_value = get_os_env('ENV')
+                modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
+                                                env_value[LcConst.BACK_END_API_PORT],
+                                                env_value[LcConst.BACK_END_API_URI])
                 modeler_rest.lc_create_run()
                 accuinsight.off_autolog()
-            
-        
+
+                
         class visualCallbacks(keras.callbacks.Callback):
             def __init__(self, x_validation=None, y_validation=None):
                 self.x_val = x_validation
@@ -436,23 +436,47 @@ class accuinsight(object):
         gorilla.apply(gorilla.Patch(keras.Model, 'fit', fit, settings=settings))
 
 
-    def _send_message(metric = None, current_value = None, message = None, thresholds=None, token = None, channel_id = None):
+    def _send_message(metric=None, current_value=None, message=None, thresholds=None, token=None, channel_id=None):
+        
+        logger = logging.getLogger(__name__)
+        
         if token is not None and channel_id is not None:
+        
             if thresholds is not None:
                 try:
                     if current_value >= thresholds:
-                        msg = '[모델 학습 완료] ' + metric +'이 설정하신 thresolds: ' + str(thresholds) + '를 초과하였습니다.'
+                        msg = '[모델 학습 완료] ' + metric + '이 설정하신 thresolds: ' + str(thresholds) + '를 초과하였습니다.'
                         response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
                 except SlackApiError as e:
-                    assert e.response["error"]
+                    logger.error(f"Error message: {e}")
+                    
             elif message is not None:
                 try:
                     msg = message
                     response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
                 except SlackApiError as e:
-                    assert e.response["error"]
-            else:
-                pass
+                    logger.error(f"Error message: {e}")
+        else:
+            pass
+
+
+#    def _send_message(metric = None, current_value = None, message = None, thresholds=None, token = None, channel_id = None):
+#        if token is not None and channel_id is not None:
+#            if thresholds is not None:
+#                try:
+#                    if current_value >= thresholds:
+#                        msg = '[모델 학습 완료] ' + metric +'이 설정하신 thresolds: ' + str(thresholds) + '를 초과하였습니다.'
+#                        response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
+#                except SlackApiError as e:
+#                    assert e.response["error"]
+#            elif message is not None:
+#                try:
+#                    msg = message
+#                    response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
+#                except SlackApiError as e:
+#                    assert e.response["error"]
+#            else:
+#                pass
 
     def off_autolog():
         def stop_log(self, original, args, kwargs, unlogged_params):
