@@ -5,30 +5,31 @@ import json
 import logging
 import warnings
 from pathlib import Path
-import datetime 
+import time
+import datetime
 from collections import OrderedDict
 
 import boto3
+from hdfs3 import HDFileSystem
 import gorilla
 import pymysql
-from slack import WebClient
-from slack.errors import SlackApiError
-
+from slack_sdk import WebClient
+from slack_sdk.errors import SlackApiError
 import numpy as np
 import tensorflow
 from tensorflow.keras.callbacks import CSVLogger
 from Accuinsight.modeler.core import func, path, get
 from Accuinsight.modeler.core.func import get_time
 from Accuinsight.modeler.core.LcConst import LcConst
-from Accuinsight.modeler.core.LcConst.LcConst import RUN_NAME_TENSORFLOW, ENV_JUPYTER_HOME_DIR 
+from Accuinsight.modeler.core.LcConst.LcConst import RUN_NAME_TENSORFLOW, ENV_JUPYTER_HOME_DIR
 from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear_runs, set_model_json_path, \
     set_visual_csv_path, set_visual_json_path, set_best_model_json_path, set_best_model_h5_path, print_run_info, \
-    set_python_dependencies, set_run_name, set_model_file_path, set_prefix_path, set_shap_json_path 
+    set_python_dependencies, set_run_name, set_model_file_path, set_prefix_path, set_shap_json_path
 from Accuinsight.modeler.core.get_for_visual import roc_pr_curve, get_true_y, get_visual_info_regressor
 from Accuinsight.modeler.clients.modeler_api import LifecycleRestApi
 from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
 from Accuinsight.modeler.utils.dl_utils import delete_files_except_best, get_best_model_path
-from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook
+from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook, get_os_env
 from Accuinsight.modeler.utils.runs_utils import get_aws_info, ProgressPercentage
 from Accuinsight.modeler.core.feature_contribution import shap_value
 
@@ -38,9 +39,10 @@ logging.basicConfig(level=logging.INFO,
 warnings.filterwarnings("ignore")
 
 
+
 class accuinsight(object):
     def __init__(self):
-        self.BucketInfo = None
+        self.StorageInfo = None
         self.target_name = None
         self.save_path = None
         self.endpoint = None
@@ -48,69 +50,172 @@ class accuinsight(object):
         self.token = None
         self.channel_id = None
         self.message = None
-
-    def get_file(self, aws_json_file_name):
+        self.feature_name = None
         
-        # meta file 저장하는 폴더 만들면, 이 경로는 수정돼야 함.
-        aws_info_json_path = os.path.join(ENV_JUPYTER_HOME_DIR, aws_json_file_name)
-       
-        with open(aws_info_json_path) as jsonFile:
-            self.BucketInfo = json.load(jsonFile)
+        
+    def set_storage(self,
+                    access_key=None,     # aws-s3
+                    secret_key=None,     # aws-s3
+                    region=None,         # aws-s3
+                    bucket_name=None,    # aws-s3
+                    hdfs_uri=None,       # pipeline-hdfs
+                    file_path=None,      # common
+                    target=None,         # common
+                    save_json=False,     # common - if it is TRUE, saving a storage information as a json file.
+                    save_path=None       # common - if save_path is not None, the json file is saved in this path.
+                   ):
+        
+        self.storage_info_from_set = OrderedDict()
+        
+        if target is None:
+            raise ValueError('target을 입력해주시기 바랍니다.')
+        
+        ## make a directory for saving json file
+        if save_json == True:
+            save_prefix_1 = os.path.join(ENV_JUPYTER_HOME_DIR, 'runs')
+            if not os.path.isdir(save_prefix_1):
+                try:
+                    os.mkdir(save_prefix_1)
+                except:
+                    pass
+            save_prefix_2 = os.path.join(save_prefix_1, 'storage-info-json')
+            if not os.path.isdir(save_prefix_2):
+                try:
+                    os.mkdir(save_prefix_2)
+                except:
+                    pass
+        
+        ### HDFS
+        if hdfs_uri is not None:
+            storage_type = 'hdfs'
             
-        BUCKET_TYPE = self.BucketInfo['bucketType']
-        BUCKET_NAME = self.BucketInfo['bucketName']
-        FILE_PATH = self.BucketInfo['filePath']
-        FILE_NAME = self.BucketInfo['fileName']
-        #FILE_TYPE = self.BucketInfo['fileType']
-        #FILE_DELIM = self.BucketInfo['fileDelim']
-        ACCESS_KEY = self.BucketInfo['myAccessKey']
-        SECRET_KEY = self.BucketInfo['mySecretKey']
-        REGION = self.BucketInfo['region']
-        URL = self.BucketInfo['endpoint']
+            split_1 = hdfs_uri.split('//')[1].split('/')
+            split_2 = [i.split(':') for i in split_1 if ':' in i][0]
+            
+            self.storage_info_from_set['host'] = split_2[0]
+            self.storage_info_from_set['port'] = split_2[1]
+            self.storage_info_from_set['filePath'] = os.path.join('/', *split_1[1:])
+            self.storage_info_from_set['target'] = target
         
-        ## define the target name 
-        self.target_name = self.BucketInfo['target']
-
-        ## path for saving data
-        save_dir = os.path.join(ENV_JUPYTER_HOME_DIR, 'data_from_AWS')
-        if os.path.exists(save_dir) == False:
-            os.mkdir(save_dir)
+        ### AWS - S3
         else:
-            pass
+            storage_type = 's3'
+            
+            self.storage_info_from_set['myAccessKey'] = access_key
+            self.storage_info_from_set['mySecretKey'] = secret_key
+            self.storage_info_from_set['endpoint'] = 's3.' + region + '.amazonaws.com'
+            self.storage_info_from_set['region'] = region
+            self.storage_info_from_set['bucketType'] = 's3'
+            self.storage_info_from_set['bucketName'] = bucket_name
+            self.storage_info_from_set['filePath'] = file_path
+            self.storage_info_from_set['fileName'] = file_path.split('/')[-1]
+            self.storage_info_from_set['target'] = target
         
-        self.save_file_name = FILE_NAME.split('.')[0] + '_' + str(datetime.datetime.now().date()).replace('-', '') + '.' + FILE_NAME.split('.')[1]
-        self.save_path = os.path.join(save_dir, self.save_file_name)
+        if save_json == True:
+            if save_path is None:
+                timestamp = str(datetime.datetime.now().date()).replace('-', '')
+                dir_list = os.listdir(save_prefix_2)
+                num = len([True for i in dir_list if timestamp in i])
+                save_path = timestamp + '-' + str(num+1).zfill(2) + '-' + storage_type + '-connection-info' + '.json'
+                
+            else:    # save_path is not None
+                pass
+                
+            storage_info_full_path = os.path.join(save_prefix_2, save_path)
+            with open(storage_info_full_path, 'w', encoding='utf-8') as save_file:
+                json.dump(self.storage_info_from_set, save_file, indent='\t')
+                
+        if save_json == False and save_path is not None:
+            raise ValueError("'save_json=True'로 설정하시기 바랍니다.")
+        
+        
+    def get_file(self, storage_json_file_name=None):
 
-        # endpoint
-        pre_url = 'https://' + BUCKET_NAME + '.' + URL
-        self.endpoint = os.path.join(pre_url, FILE_PATH)
+        if storage_json_file_name is not None:
+        
+            # meta file 저장하는 폴더 만들면, 이 경로는 수정돼야 함.
+            storage_info_json_path = os.path.join(ENV_JUPYTER_HOME_DIR, storage_json_file_name)
+       
+            with open(storage_info_json_path) as jsonFile:
+                self.StorageInfo = json.load(jsonFile)
+        
+        else:   # storage_json_file_name == None, when using set_storage() method.
+            self.StorageInfo = self.storage_info_from_set
+            
+        ## define the target name
+        self.target_name = self.StorageInfo['target']
+            
+        ### HDFS
+        if 'host' in self.StorageInfo.keys():
+            
+            ## path for saving data
+            save_dir = os.path.join(ENV_JUPYTER_HOME_DIR, 'data_from_hdfs')
+            if os.path.exists(save_dir) == False:
+                os.mkdir(save_dir)
+            else:
+                pass
+            
+            split_list = list(filter(None, self.StorageInfo['filePath'].split('/')))
+            
+            HOST = self.StorageInfo['host']
+            PORT = int(self.StorageInfo['port'])
+            FILE_PATH = self.StorageInfo['filePath']
+            FILE_DIR = '/'
+            for i in split_list[:len(split_list)-1]:
+                FILE_DIR = os.path.join(FILE_DIR, i)
+            FILE_NAME = split_list[-1]
+            
+            self.save_file_name = FILE_NAME.split('.')[0] + '_' + str(datetime.datetime.now().date()).replace('-', '') + '.' + FILE_NAME.split('.')[1]
+            self.save_path = os.path.join(save_dir, self.save_file_name)
+            self.endpoint = os.path.join('hdfs://', HOST, self.StorageInfo['port']) + FILE_PATH
+            
+            hdfs = HDFileSystem(host = HOST, port = PORT)
+            hdfs.ls(FILE_DIR)
 
-        client = boto3.client(BUCKET_TYPE,
-                              aws_access_key_id=ACCESS_KEY,
-                              aws_secret_access_key=SECRET_KEY,
-                              region_name=REGION)
+            sys.stdout.write('%s %s %s' % ('Downloading file...', FILE_NAME, '\n'))
+            time.sleep(1)
+            hdfs.get(FILE_PATH, self.save_path)
+            logging.info(self.save_path)
+        
+        ### AWS-S3
+        else:
+            BUCKET_TYPE = self.StorageInfo['bucketType']
+            BUCKET_NAME = self.StorageInfo['bucketName']
+            FILE_PATH = self.StorageInfo['filePath']
+            FILE_NAME = self.StorageInfo['fileName']
+            #FILE_TYPE = self.StorageInfo['fileType']
+            #FILE_DELIM = self.StorageInfo['fileDelim']
+            ACCESS_KEY = self.StorageInfo['myAccessKey']
+            SECRET_KEY = self.StorageInfo['mySecretKey']
+            REGION = self.StorageInfo['region']
+            URL = self.StorageInfo['endpoint']
 
-        transfer = boto3.s3.transfer.S3Transfer(client)
+            ## path for saving data
+            save_dir = os.path.join(ENV_JUPYTER_HOME_DIR, 'data_from_aws')
+            if os.path.exists(save_dir) == False:
+                os.mkdir(save_dir)
+            else:
+                pass
+        
+            self.save_file_name = FILE_NAME.split('.')[0] + '_' + str(datetime.datetime.now().date()).replace('-', '') + '.' + FILE_NAME.split('.')[1]
+            self.save_path = os.path.join(save_dir, self.save_file_name)
 
-        progress = ProgressPercentage(client, BUCKET_NAME, FILE_PATH)
+            # endpoint
+            pre_url = 'https://' + BUCKET_NAME + '.' + URL
+            self.endpoint = os.path.join(pre_url, FILE_PATH)
 
-        sys.stdout.write('%s %s %s' % ('Downloading file...', FILE_NAME, '\n'))
-        transfer.download_file(BUCKET_NAME, FILE_PATH, self.save_path, callback=progress)
-        logging.info(self.save_path)
-    
-   # def get_feature_name(self):
-   #     fileType = self.BucketInfo['fileName'].split('.')[0]
-   #     if fileType == 'json':
-   #         with open(self.save_path) as jsonFile:
-   #             json_data = json.load(jsonFile)
-   #         self.feature_name = json_data.keys()
-   #     elif fileType == 'csv':
-   #         self.feature_name = list(pd.read_csv(self.save_path).columns)
-   #         if 'Unnamed: 0' in feature_list:
-   #             self.feature_name.remove('Unnamed: 0')
-   #     else:
-   #         logging.info('현재 지원하는 데이터 형식은 json(key: feature name)과 csv입니다.\n feature name을 가져올 수 없는 경우, [feature_1, feature_2, ...]과 같이 임의로 할당합니다.')
-   #     return(self.feature_name)
+            client = boto3.client(BUCKET_TYPE,
+                                  aws_access_key_id=ACCESS_KEY,
+                                  aws_secret_access_key=SECRET_KEY,
+                                  region_name=REGION)
+
+            transfer = boto3.s3.transfer.S3Transfer(client)
+
+            progress = ProgressPercentage(client, BUCKET_NAME, FILE_PATH)
+
+            sys.stdout.write('%s %s %s' % ('Downloading file...', FILE_NAME, '\n'))
+            transfer.download_file(BUCKET_NAME, FILE_PATH, self.save_path, callback=progress)
+            logging.info(self.save_path)
     
     def set_slack(self, token=None, channel_id=None):
         self.token = token
@@ -123,9 +228,24 @@ class accuinsight(object):
             self.message = message
             self.thresholds = thresholds
 
+    def set_features(self, feature_names):
+        
+        if str(type(feature_names)) == "<class 'list'>":
+            self.feature_name = feature_names
+        else:
+            try:
+                self.feature_name = feature_names.tolist()
+            except:
+                pass
+            
+            try:
+                self.feature_name = list(feature_names)
+            except AttributeError:
+                raise ValueError('입력값으로 list 또는 pandas.core.indexes.base.Index를 사용하시기 바랍니다.')
+                
+
     def autolog(self, tag=None, best_weights = False, model_monitor=False):
-        global description, endpoint, var_model_file_path, message, thresholds, token, channel_id, best_weights_on, \
-               feature_name, run_id, shap_on
+        global description, endpoint, var_model_file_path, message, thresholds, token, channel_id, best_weights_on, run_id, shap_on, feature_name
         description = tag
         endpoint = self.endpoint
         message = self.message
@@ -139,17 +259,17 @@ class accuinsight(object):
             best_weights_on = False
         if model_monitor == True:
             shap_on = True
-            try:
-                feature_name = get.feature_name(self.save_path, self.BucketInfo, self.target_name)
-            except:
-                feature_name = None      # when user did not using <get_file> function     
+            if self.feature_name is None:
+                try:
+                    feature_name = get.feature_name(self.save_path, self.StorageInfo, self.target_name)
+
+                except:
+                    pass      # when user did not use <get_file> or <set_feature> function
+            else:
+                feature_name = self.feature_name
         else:
             shap_on = False
         run_id = None
-        
-        
-        
-#         print(feature_name)   ####################################################
 
         is_notebook = is_in_ipython()
         if is_notebook == True:
@@ -266,17 +386,11 @@ class accuinsight(object):
                                       'skipping.' % (self.monitor), RuntimeWarning)
                         else:
                             if self.monitor_op(current, self.best):
-#                                 if self.verbose > 0:
-#                                     print('\n\nEpoch %05d: %s improved from %0.5f to %0.5f,'
-#                                           ' storing weights.\n'
-#                                           % (epoch + 1, self.monitor, self.best, current))
                                 self.best = current
                                 self.best_epochs = epoch + 1
                                 self.best_weights = self.model.get_weights()
                             else:
                                 pass
-#                                 if self.verbose > 0:
-#                                     print('\n\nEpoch %05d: %s did not improve\n' % (epoch + 1, self.monitor))
 
                     self.current_value = current
     
@@ -347,9 +461,10 @@ class accuinsight(object):
                                           thresholds=thresholds,
                                           token=token,
                                           channel_id=channel_id)
-                modeler_rest = LifecycleRestApi(LcConst.BACK_END_API_URL,
-                                                LcConst.BACK_END_API_PORT,
-                                                LcConst.BACK_END_API_URI)
+                env_value = get_os_env('ENV')
+                modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
+                                                env_value[LcConst.BACK_END_API_PORT],
+                                                env_value[LcConst.BACK_END_API_URI])
                 modeler_rest.lc_create_run()
                 accuinsight._off_autolog()
 
@@ -388,15 +503,12 @@ class accuinsight(object):
                 self.trainX = trainX
                 self.trigger = shap_on
                 self.run_id = run_id
-                #if feature_name is not None:
-                #    self.feature_name = feature_name
-                #else:
-                self.feature_name = feature_name
-                
+                self.feature_name_in_shap = feature_name
+
 
             def on_train_end(self, logs={}):
                 if self.trigger:
-                    self.shap_value = shap_value(self.model, self.trainX, self.feature_name)
+                    self.shap_value = shap_value(self.model, self.trainX, self.feature_name_in_shap)
                     
 #                     func.insertDB(self.shap_value, 2)  # 수정: 2 -> self.run_id
                     
@@ -410,7 +522,7 @@ class accuinsight(object):
                         json.dump(self.shap_value, save_file, indent='\t')
 
                 else:
-                    pass    
+                    pass
                     
         def run_and_log_function(self, original, x, y, kwargs):
             dict_path = path.get_file_path(self, usedFramework='tensorflow')
@@ -428,10 +540,8 @@ class accuinsight(object):
             if 'x':
                 x_train = x
             if shap_on:
-#                print('using model_monitor_1')
                 get_shap = shapCallbacks(x_train, feature_name, run_id, trigger = shap_on)
             else:
-#                print('not using model_monitor_1')
                 pass
             
             ''' save json for visualization '''
@@ -488,20 +598,25 @@ class accuinsight(object):
         gorilla.apply(gorilla.Patch(tensorflow.keras.Model, 'fit', fit, settings=settings))
 
     def _send_message(metric=None, current_value=None, message=None, thresholds=None, token=None, channel_id=None):
+        
+        logger = logging.getLogger(__name__)
+        
         if token is not None and channel_id is not None:
+        
             if thresholds is not None:
                 try:
                     if current_value >= thresholds:
                         msg = '[모델 학습 완료] ' + metric + '이 설정하신 thresolds: ' + str(thresholds) + '를 초과하였습니다.'
                         response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
                 except SlackApiError as e:
-                    assert e.response["error"]
+                    logger.error(f"Error message: {e}")
+                    
             elif message is not None:
                 try:
                     msg = message
                     response = WebClient(token=token).chat_postMessage(channel=channel_id, text=msg)
                 except SlackApiError as e:
-                    assert e.response["error"]
+                    logger.error(f"Error message: {e}")
         else:
             pass
 
@@ -518,3 +633,4 @@ class accuinsight(object):
 
         settings = gorilla.Settings(allow_hit=True, store_hit=True)
         gorilla.apply(gorilla.Patch(tensorflow.keras.Model, 'fit', fit, settings=settings))
+
