@@ -1,6 +1,7 @@
 import os
 import itertools
-from Accuinsight.modeler.core import get
+
+from Accuinsight.modeler.core import get, metricsLightgbm
 from Accuinsight.modeler.core.LcConst.LcConst import RUN_ROOT_PATH
 from Accuinsight.modeler.core.Run.RunInfo.RunInfo import get_or_create_run_directory
 
@@ -14,8 +15,6 @@ def get_file_path(model_name, root_path = RUN_ROOT_PATH, **kwargs):
     :root_path:
     :kwargs: DL framework를 구분하기 위한 변수 입력, (eg. usedFramework = 'keras')
 
-    # [참고] visualization을 위한 데이터 생성이 모델과 분석 문제에 따라 달라지므로 주의
-      myshare 주소
     """
 
     
@@ -68,7 +67,15 @@ def get_file_path(model_name, root_path = RUN_ROOT_PATH, **kwargs):
 
         path_list = [json_path_model, csv_path_visual, json_path_visual, json_path_shap]   # DL
 
-
+    elif 'lightgbm' in get.model_type(model_name):
+        metric_lightgbm = get.from_model(model_name).all_params()['metric']
+        if metric_lightgbm in metricsLightgbm.CLASSIFICATION:
+            path_list = [json_path_model, json_path_visual, json_path_shap]       # lightgbm & Classification
+        elif metric_lightgbm in metricsLightgbm.REGRESSION:
+            path_list = [json_path_model, json_path_shap]                         # lightgbm & Regression
+        else:
+            pass
+        
     elif 'Classifier' in get.model_type(model_name) or 'Logistic' in get.model_type(model_name) or 'SVC' in get.model_type(model_name) or 'NB' in get.model_type(model_name):
         path_list = [json_path_model, json_path_visual, json_path_shap]       # ML & Classification
 
